*************************************************************************
*  MACRO:  'buttfly'   optimized general butterfly radix 2 for 320C5xx  *
*                                                                       *
*  version 0.99    from Manfred Christ   update:  2. May 94             *
*  version 1.00    by Mansoor A. Chishtie        22  Nov 96             *
*                                                                       *
*  Update: Modified Butterfly now takes only 8 cycles. One butterfly is *
*           done outside the RPTB loop.                                 *
*                                                                       *
*                                                                       *
*                                                                       *
*                                                                       *
*  ASM = -1  / SXM=1 / BK=0 / FRCT=1 / BK=0                             *
*                                                                       *
*      Cycles:                                                          *
*      Prolog code: number of words/number of instr/cycle=19            *
*                    (19 words includes the RPTB and first step of      *
*                     butterfly)                                        *
*      Butterfly  : 8 * (num-2) cycles                                  *
*      Epilog code: number of words/number of cycles=2                  *
*                                                                       *
*        CALCULATE (startup code):                                      *
*        QR'=(PR-QR)/2      <-AR2                                       *
*        PR'=(PR+QR)/2      <-AR3+1                                     *
*        PI'=(PI+QI)/2      <-AR2                                       *
*        PI'=(PI-QI)/2      <-AR3+1                                     *
*************************************************************************
*                                                                       *
*  Def.: AR2 -> QR   (input)  AR2 -> QR+1     (output)                  *
*  Def.: AR3 -> PR   (input)  AR3 -> PR+1     (output)                  *
*  Def.: AR4 -> Cxxx (input)  AR4 -> Cxxx+1   (output) --> WR=cosine    *
*  Def.: AR5 -> Sxxx (input)  AR5 -> Sxxx+1   (output) --> WI=sine      *
*                                                                       *
*        PR' = (PR+(QR*WR+QI*WI))/2       WR=COS(W)   WI=SIN(W)         *
*        PI' = (PI+(QI*WR-QR*WI))/2  = (PI-(QR*WI-QI*WR))/2             *
*        QR' = (PR-(QR*WR+QI*WI))/2                                     *
*        QI' = (PI-(QI*WR-QR*WI))/2  = (PI+(QR*WI-QI*WR))/2             *
*************************************************************************
buttfly2 .macro num	     ;		   (contents of register after exec.)

      .asg  AR2, P
      .asg  AR3, Q
      .asg  AR4,WR
      .asg  AR5,WI
      
;X    STM  #-2,AR0          ; index = -2
      STM  #:num:-3,BRC     ; execute startup + num-3 times general BUTTFLY
;                                                         AR2  AR3  AR4  AR5
; takes 17 words-/cycles (including RPTB)                 ---  ---  ---  ---
      LD   *P,16,A        ;A  :=  PR                      PR   QR   WR   WI
      SUB  *Q,16,A,B      ;B  :   PR-QR                   PR   QR   WR   WI
      ST   B,*Q     ;<<ASM;QR':= (PR-QR)/2
||    ADD  *Q+,B          ;B  := (PR+QR)                  PR   QI   WR   WI
      ST   B,*P+    ;<<ASM;PR':= (PR+QR)/2
||    LD   *Q,A     ;<<16 ;A  :=  QI                      PI   QI   WR   WI
      ADD  *P,16,A,B      ;B  := (PI+QI)                  PI   QI   WR   WI
      ST   B,*P     ;<<ASM;PI':= (PI+QI)/2
||    SUB  *P+,B          ;B  := (PI-QI)                  PR+1 QR   WR   WI
      STH   B,ASM,*Q+     ;QI':= (PI-QI)/2                PR+1 QR+1 WR   WI

      MPY   *WR,*Q+,A     ;A  := QR*WR                    PR+1 QI+1 WR   WI
      MAC   *WI+,*Q-,A    ;A  := (QR*WR+QI*WI) || T=WI    PR+1 QR+1 WR   WI+1
      ADD   *P,16,A,B     ;B  := (PR+(QR*WR+QI*WI))       PR+1 QR+1 WR   WI+1
      ST    B,*P    ;<<ASM;PR':= (PR+(QR*WR+QI*WI))/2     
||    SUB   *P+,B         ;B  := (PR-(QR*WR+QI*WI))       PI+1 QR+1 WR   WI+1
      ST    B,*Q    ;<<ASM;QR':= (PR-(QR*WR+QI*WI))/2
||    MPY   *Q+,A         ;A  := QR*WI     [t=WI]         PI+1 QI+1 WR   WI+1
      MAS   *WR+,*Q,A     ;A  := (   (QR*WI-QI*WR))       PI+1 QI+1 WR+1 WI+1


      RPTBD end2?-1	   ;delayed block repeat
      ST    A,*Q+         ;dummy write
||    SUB   *P,B          ;B  := (PI-(QR*WI-QI*WR))       PI+1 QR+2 WR+1 WI+1
      ST    B,*P    ;<<ASM;PI':= (PI-(QR*WI-QI*WR))/2
||    ADD   *P+,B         ;B  := (PI+(QR*WI-QI*WR))       PR+2 QR+2 WR+1 WI+1
;                                                             
; Butterfly kernal with 8 instructions / 8 cycles
;
      MPY   *WR,*Q+,A     ;A  :=  QR*WR                   PR+2 QI+2 WR+1 WI+1
      MAC   *WI+,*Q+0%,A  ;A  := (QR*WR+QI*WI) || T=WI    PR+2 QI+1 WR+1 WI+2
      ST    B,*Q+   ;<<ASM;QI':= (PI+(QR*WI-QI*WR))/2
||    ADD   *P,B          ;B  := (PR+(QR*WR+QI*WI))       PR+2 QR+2 WR+1 WI+2
      ST    B,*P    ;<<ASM;PR':= (PR+(QR*WR+QI*WI))/2
||    SUB   *P+,B         ;B  := (PR-(QR*WR+QI*WI))       PI+2 QR+2 WR+1 WI+2
      ST    B,*Q    ;<<ASM;QR':= (PR-(QR*WR+QI*WI))/2
||    MPY   *Q+,A         ;A  := QR*WI     [t=WI]         PI+2 QI+2 WR+1 WI+2
      MAS   *WR+,*Q,A     ;A  := (   (QR*WI-QI*WR))       PI+2 QI+2 WR+2 WI+2
      ST    A,*Q+         ;dummy write
||    SUB   *P,B          ;B  := (PI-(QR*WI-QI*WR))       PI+2 QR+3 WR+2 WI+2
      ST    B,*P    ;<<ASM;PI':= (PI-(QR*WI-QI*WR))/2
||    ADD   *P+,B         ;B  := (PI+(QR*WI-QI*WR))       PR+3 QR+3 WR+2 WI+2
end2?
      MAR   *Q-
      STH   B,ASM,*Q+     ;QI':= (PI+(QR*WI-QI*WR))/2     PR+3 QR+3 WR+2 WI+2


      .endm
      

