         .file      "c5xx0256.asm"
         .title     "0256 point DIT Radix-2, Complex FFT,TMS320C5xx"
         .width     120
N2	 .set	    256   ; NUMBER OF POINTS FOR FFT
****************************************************************************
*
*   256  - POINT COMPLEX, RADIX-2 DIF FFT WITH THE TMS320C5xx / LOOPED CODE
*   -----------------------------------------------------------------------
*
* THE PROGRAM IS BASED ON THE BOOK 'DIGITAL SIGNAL PROCESSING APPLICATIONS'
* FROM TEXAS INSTRUMENTS P. 69. IT IS OPTIMIZED FOR THE TMS320C5x INCLUDING
* BIT REVERSAL ADDRESSING MODE.
*
*    WRITTEN BY: MANFRED CHRIST  REVISION: 0.99    2. May  94
*
*    COPYRIGHT TEXAS INSTRUMENTS INC. 1994
*
****************************************************************************
*
*    USED  REGISTER: INDX,AR0..AR5,A,B,T,ST1,BRC, 1 Stacklevel
*
*    PROGRAM MEMORY:  --- WORDS ('FFTLEN') WITHOUT INITIALIZATION
*
*    COEFFICIENTS  :   16 BITS  (Q15 Format)  SCALING:   1/2^10
*
*    CYCLES	   : 9728 CYLCES (from 'callfft' to 'wait')
*
*    MEMORY USAGE: FFT PROGRAM    [ 0F000H -  0FXXXH ]
*                  INPUT DATA     [ 01000H -  011FFH ]
*                  DATA WORKSPACE [ 00800H -  009FFH ]
*                  TWIDDLES       [ 00080H -  005ECH ]
*
*    INPUT DATA AT ADDRESS 0800H-09FFH:
*    ----------------------------------
*    THE DATA ARE STORED IN 'input' IN THE SEQUENCE: X(0),X(1),...,X(255)
*                                                    Y(0),Y(1),...,Y(255)
*
*    INPUT DATA AT ADDRESS 01000h-011FFh:
*    ------------------------------------
*    THE DATA ARE STORED IN section 'fftdata' IN THE SEQUENCE:
*    X(0),Y(0),X(1),Y(1),... ... ,X(255),Y(255)
****************************************************************************
*
*    THIS PROGRAM INCLUDES FOLLOWING FILES:
*    --------------------------------------
*    'TWIDDLES.Q15' contains all twiddle factors in Q15 format (sine,cosine)
*    'C5XXRAD2.ASM' consists of all macros for all FFTs
*    'INIT-FFT.ASM' for initialisation
****************************************************************************
	.include initfft2.asm
	.sect	 "fftprog2"
;    FFT PROGRAMME executes the DIT complex radix 2 algorithm
FFT2:
	ld    #0,DP           ; DP = Page 0
        stm   #STACK+stacksize,sp
	.if   N2>4
        ld    #SIN45,DP
        .bss  SIN45,1
        st    #5a82h,SIN45
        .endif
;
; EXECUTE THE FFT
;
        stm   #000010101011110b,ST1    ; ASM=-2 , FRACT=1, sbx=1
                                       ; shift 1 from Product to A,B
	combo5x2		      ; FFT CODE for STAGES 1 and 2
	stage32 		      ; MACRO WITH CODE FOR STAGE 3
        stdmacro 4,16,8,16,sin4,cos4  ; stage,outloopcnter,loopcnter,index
        stdmacro 5,8,16,32,sin5,cos5  ; stage,outloopcnter,loopcnter,index
        stdmacro 6,4,32,64,sin6,cos6  ; stage,outloopcnter,loopcnter,index
        stdmacro 7,2,64,128,sin7,cos7 ; stage,outloopcnter,loopcnter,index
        laststag 8,sin8,cos8          ; MACRO WITH CODE FOR STAGE 8
FFTLEN2: ret
        .end
