         .file      "c5xx1024.asm"
         .title     "1024 point DIT Radix-2, Complex FFT,TMS320C5xx"
         .width     120
N        .set       1024  ; NUMBER OF POINTS FOR FFT
****************************************************************************
*
*  1024  - POINT COMPLEX, RADIX-2 DIF FFT WITH THE TMS320C5xx / LOOPED CODE
*  ------------------------------------------------------------------------
*
* THE PROGRAM IS BASED ON THE BOOK 'DIGITAL SIGNAL PROCESSING APPLICATIONS'
* FROM TEXAS INSTRUMENTS P. 69. IT IS OPTIMIZED FOR THE TMS320C5x INCLUDING
* BIT REVERSAL ADDRESSING MODE.
*
*    WRITTEN BY: MANFRED CHRIST  REVISION: 0.99    2. May  94
*
*    COPYRIGHT TEXAS INSTRUMENTS INC. 1994
*
****************************************************************************
*
*    USED  REGISTER: INDX,AR0..AR5,A,B,T,ST1,BRC, 1 Stacklevel
*
*    PROGRAM MEMORY:  --- WORDS ('FFTLEN') WITHOUT INITIALIZATION
*
*    COEFFICIENTS  :   16 BITS  (Q15 Format)  SCALING:   1/2^12
*
*    CYCLES	   : 47952 CYLCES (from 'callfft' to 'wait')
*
*    MEMORY USAGE: FFT PROGRAM    [ 0F000H -  0FXXXH ]
*                  INPUT DATA     [ 01000H -  017FFH ]
*                  DATA WORKSPACE [ 00800H -  00FFFH ]
*                  TWIDDLES       [ 00080H -  005ECH ]
*
*    INPUT DATA AT ADDRESS 0800H-0FFFH:
*    ----------------------------------
*    THE DATA ARE STORED IN 'input' IN THE SEQUENCE: X(0),X(1),...,X(1023)
*                                                    Y(0),Y(1),...,Y(1023)
*
*    INPUT DATA AT ADDRESS 01000h-017FFh:
*    ------------------------------------
*    THE DATA ARE STORED IN section 'fftdata' IN THE SEQUENCE:
*    X(0),Y(0),X(1),Y(1),... ... ,X(1023),Y(1023)
****************************************************************************
*
*    THIS PROGRAM INCLUDES FOLLOWING FILES:
*    --------------------------------------
*    'TWIDDLES.Q15' contains all twiddle factors in Q15 format (sine,cosine)
*    'C5XXRAD2.ASM' consists of all macros for all FFTs
*    'INIT-FFT.ASM' for initialisation
****************************************************************************
        .include init-fft.asm
        .sect    "fftprog"
;    FFT PROGRAMME executes the DIT complex radix 2 algorithm
FFT:    
	ld    #0,DP           ; DP = Page 0
        stm   #STACK+stacksize,sp
        .if   N>4
        ld    #SIN45,DP
        .bss  SIN45,1
        st    #5a82h,SIN45
        .endif
;
; EXECUTE THE FFT
;
        stm   #000010101011110b,ST1    ; ASM=-2 , FRACT=1, sbx=1
                                       ; shift 1 from Product to A,B
	combo5xx                       ; FFT CODE for STAGES 1 and 2
        stage3                         ; MACRO WITH CODE FOR STAGE 3
        stdmacro 4,64,8,16,sin4,cos4   ; stage,outloopcnter,loopcnter,index
        stdmacro 5,32,16,32,sin5,cos5  ; stage,outloopcnter,loopcnter,index
        stdmacro 6,16,32,64,sin6,cos6  ; stage,outloopcnter,loopcnter,index
        stdmacro 7,8,64,128,sin7,cos7  ; stage,outloopcnter,loopcnter,index
        stdmacro 8,4,128,256,sin8,cos8 ; stage,outloopcnter,loopcnter,index
        stdmacro 9,2,256,512,sin9,cos9 ; stage,outloopcnter,loopcnter,index
        laststag 10,sina,cosa          ; MACRO WITH CODE FOR STAGE 10
FFTLEN: ret
        .end
