;
; file: c5xxrad2.asm    updated: 11/22/96  version 1.0 Mansoor A. Chishtie
;
; CONTAINS ALL MACROS FOR THE C54X COMPLEX RADIX 2 DIT FFT
;
***************************************************************************
*                                                                         *
* THE MACRO 'combo5xx' PERFORMS FOLLOWING CALCULATIONS:                  *
*                                                                         *
* R1  := [(R1+R2)+(R3+R4)]/4    INPUT               OUTPUT                *
* R2  := [(R1-R2)+(I3-I4)]/4    ------------------  ------------------    *
* R3  := [(R1+R2)-(R3+R4)]/4    AR0 =  7                                  *
* R4  := [(R1-R2)-(I3-I4)]/4    AR1 -> R1,I1        AR1 - > R5,I5         *
* I1  := [(I1+I2)+(I3+I4)]/4    AR2 -> R2,I2        AR2 - > R6,I6         *
* I2  := [(I1-I2)-(R3-R4)]/4    ARP-> AR3 -> R3,I3  ARP - > AR3 - > R7,I7 *
* I3  := [(I1+I2)-(I3+I4)]/4    AR4 -> R4,I4        AR4 - > R8,I8         *
* I4  := [(I1-I2)+(R3-R4)]/4                                              *
*                                                                         *
* For a 16-point Radix 2 complex FFT the Macro 'combo5xx' has to be       *
* repeated N/4 times (e.g. 4  times for a  16 point FFT).                 *
*                                                                         *
* Cycles for combo5xx: 29 instructions/cycles                       *
*                                                                         *
***************************************************************************
;
        .include combo5xx.asm    ; orgversion
        .include stage3.asm      ; orgversion
    ;   .include combo5xa.asm    ; new version
    ;   .include stage3a.asm     ; new version

      .include buttfly.asm

laststag .macro  stage,sin,cos
         .global STAGE:stage:,end?
STAGE:stage: .set $
      stm   #DATA,ar2        ; ar2 -> DATA
      stm   #DATA+N,ar3      ; ar3 -> DATA+(offset=N)
      stm   #cos,ar4         ; start of cosine in stage 'stg'
      stm   #sin,ar5         ; start of sine in stage   'stg'
      buttfly N/2            ; execute N/2 butterflies
      .endm


stdmacro .macro  stage,l1,l2,idx,sin,cos
         .global STAGE:stage:,end?
STAGE:stage: .set $
      stm   #DATA,ar2        ; ar2 -> DATA
      stm   #DATA+idx,ar3    ; ar3 -> DATA+(offset=idx)
      stm   #l1-1,ar1        ; outer loop counter
      stm   #cos,ar6         ; start of cosine in stage 'stg'
      stm   #sin,ar7         ; start of sine in stage 'stg'

loop? mvmm  ar6,ar4          ; start of cosine in stage 'stg'
      mvmm  ar7,ar5          ; start of sine in stage 'stg'

      buttfly l2             ; execute l2 butterflies

      mar   *+ar2(idx)
      banzd loop?,*ar1-
      mar   *+ar3(idx)
      .endm
