***************************************************************************
*                                                                         *
* Modified by: Mansoor A. Chishtie Nov 1996                               *
* Comments: Removed separate bit-reversal stage and combined it with      *
*           first two stages.                                             *
*                                                                         *
* Cycles:   Epilog code: 20 cycles including RPTBD			  *
*	    Looped Code: 29 cycles per iteration * N/4 iterations	  *
*	    Prolog code: 1 cycle					  *
*									  *
* Alignment:DATA buffer must be aligned to 2*N address.                   *
*           INPUT buffer must be aligned to 4*N address.                  *
*                                                                         *
*                                                                         *
* R1  := [(R1+R2)+(R3+R4)]/4                                              *
* R2  := [(R1-R2)+(I3-I4)]/4                                              *
* R3  := [(R1+R2)-(R3+R4)]/4                                              *
* R4  := [(R1-R2)-(I3-I4)]/4                                              *
* I1  := [(I1+I2)+(I3+I4)]/4                                              *
* I2  := [(I1-I2)-(R3-R4)]/4                                              *
* I3  := [(I1+I2)-(I3+I4)]/4                                              *
* I4  := [(I1-I2)+(R3-R4)]/4                                              *
*                                                                         *
***************************************************************************

combo5xx .macro                     ; REPEAT MACRO 'combo5xx': N/4 times
         .global STAGE1,BITREV,COMBO,end?
;
STAGE1: stm     #INPUT+0,ar2        ; pointer to R1,I1
        stm     #INPUT+(N/2),ar3    ; pointer to R2,I2
        stm     #INPUT+(N/4),ar4    ; pointer to R3,I3
        stm     #DATA+3,ar6         ; pointer to R1',I1' (+3 offset)
        stm     #DATA+4+3,ar7       ; pointer to R3',I3' (+3 offset)

        stm     #N,ar0              ; used to move b/w Rn and In
        stm     #2*N,BK             ; INPUT buffer should be aligned to 4N

        .if     N>4
        stm     #N/4-1,BRC          ; execute N/4 times 'combo5xx'
        rptbd   end?
        .endif
        stm     #(INPUT+(3*N/4)),ar5; pointer to R4,I4     
                                    ;                               AR2 AR3 AR4 AR5
                              ;                                     --- --- --- ---
COMBO   add     *ar2+0%,*ar3+0%,A   ; A  := (R1+R2)                 I1  I2  R3  R4       
        add     *ar4+0%,*ar5+0%,B   ; B  := (R3+R4)                 I1  I2  I3  I4       
        add     B,A                 ; A  := (R1+R2) + (R3+R4)                            
        sth     A,ASM,*+ar6(-3)     ; R1':=((R1+R2) + (R3+R4))/4                         
        sub     B,1,A               ; A  := (R1+R2) - (R3+R4)                            
        sth     A,ASM,*+ar7(-3)     ; R3':=((R1+R2) - (R3+R4))/4                         
                                                                                     
        add     *ar2+0%,*ar3+0%,A   ; A  := (I1+I2)                 R1  R2  I3  I4       
        add     *ar4, *ar5, B       ; B  := (I3+I4)                                      
        add     B,A                 ; A  := (I1+I2) + (I3+I4)                            
        sth     A,ASM,*+ar6         ; I1':= (I1+I2) + (I3+I4)/4                          
        sub     B,1,A               ; A  := (I1+I2) - (I3+I4)                            
        sth     A,ASM,*+ar7         ; I3':= (I1+I2) - (I3+I4)/4                          
                                                                                     
        sub     *ar2+0%,*ar3+0%,A   ; A  := (R1-R2)                 I1  I2  I3  I4       
        sub     *ar4+0%,*ar5+0%,B   ; B  := (I3-I4)                 I1  I2  R3  R4       
        add     B,A                 ; A  := (R1-R2) + (I3-I4)                            
        sth     A,ASM,*+ar6         ; R2':= (R1-R2) + (I3-I4)/4                          
        sub     B,1,A               ; A  := (R1-R2) - (I3-I4)                            
        sth     A,ASM,*+ar7         ; R4':= (R1-R2) - (I3-I4)                            
                                                                                     
        sub     *ar2+0%,*ar3+0%,A   ; A  := (I1-I2)                 R1  R2  R3  R4       
        sub     *ar4+,*ar5+,B       ; B  := (R3-R4)                 R1  R2  R3* R4*      
        sub     B,A                 ; A  := (I1-I2) - (R3-R4)                            
        sth     A,ASM,*+ar6         ; I2':= (I1-I2) - (R3-R4)/4                          
        add     B,1,A               ; A  := (I1-I2) + (R3-R4)                            
        sth     A,ASM,*+ar7         ; I4':= (I1-I2) + (R3-R4)/4                          
                                                                                     
        ld      *ar2+,A             ; dummy instruction             R1* R2* R3* R4*      
||      mac     *ar3+,B                                                              
        mar     *ar6+0B             ; update AR6
end?    mar     *ar7+0B             ; update AR7                                                 

        stm     #0,BK               ; Disable circular addressing
           .endm
