************************************************************
**                                                        **
**   Interrupt Service Pointer Table                      **
**                                                        **
**   This code allows indirect vectoring of interrupts    **
**   based on the addresses loaded in RAM block B2.       **
**   This allows the definition of addresses to be        **
**   deferred until the RAM resident code is assembled.   **
**                                                        **
**   WRITTEN BY:  Christopher R. Hansen                   **
**   DATE:        1/3/92                                  **
**                                                        **
**   Revision History                                     **
**   1.1                                                  **
**                                                        **
************************************************************

        .length 60

        .def    int0v,int1v,int2v,int3v
        .def    tintv
        .def    brint0v,bxint0v
        .def    brint1v,bxint1v
        .def    trntv,txntv
        .def    hpintv
        .def    nmintv, rstv
        .def    sint17v,sint18v,sint19v,sint20v
        .def    sint21v,sint22v,sint23v,sint24v,sint25v
        .def    sint26v,sint27v,sint28v,sint29v,sint30v

        .ref    boot

rstv    .set    60h             ; external interrupt vectors
nmintv  .set    61h             ; Nonmaskable interrupt
sint17v .set    62h             ; Software INT #17
sint18v .set    63h             ; Software INT #18
sint19v .set    64h             ; Software INT #19
sint20v .set    65h             ; Software INT #20
sint21v .set    66h             ; Software INT #21
sint22v .set    67h             ; Software INT #22
sint23v .set    68h             ; Software INT #23
sint24v .set    69h             ; Software INT #24
sint25v .set    6ah             ; Software INT #25
sint26v .set    6bh             ; Software INT #26
sint27v .set    6ch             ; Software INT #27
sint28v .set    6dh             ; Software INT #28
sint29v .set    6eh             ; Software INT #29
sint30v .set    6fh             ; Software INT #30

int0v   .set    70h             ; External INT0
int1v   .set    71h             ; External INT1
int2v   .set    72h             ; External INT2
tintv   .set    73h             ;
brint0v .set    74h             ; BSP0 Receive Int
bxint0v .set    75h             ; BSP0 Transmit Int
trntv   .set    76h             ; TDM Receive Int
txntv   .set    77h             ; TDM Transmit Int
int3v   .set    78h             ; External INT3
hpintv  .set    79h             ; HPI Interrupt
brint1v .set    7ah             ; BSP1 Receive Int
bxint1v .set    7bh             ; BSP1 Transmit Int

        .sect   "int_vect"

reset   bd      0f800h          ; 0 - power on reset
        stm     #06fh,SP
        ld      nmintv, a       ; 1 - NMI interrupt
        fbacc   a
        nop
        nop
        ld      sint17v, a      ; 2 - software interrupt #17
        fbacc   a
        nop
        nop
        ld      sint18v, a      ; 3 - external interrupt #18
        fbacc   a
        nop
        nop
        ld      sint19v, a      ; 4 - software interrupt #19
        fbacc   a
        nop
        nop
        ld      sint20v, a      ; 5 - software interrupt #20
        fbacc   a
        nop
        nop
        ld      sint21v, a      ; 6 - external interrupt #21
        fbacc   a
        nop
        nop
        ld      sint22v, a      ; 7 - software interrupt #22
        fbacc   a
        nop
        nop
        ld      sint23v, a      ; 8 - software interrupt #23
        fbacc   a
        nop
        nop
        ld      sint24v, a      ; 9 - external interrupt #24
        fbacc   a
        nop
        nop
        ld      sint25v, a      ; 10 - software interrupt #25
        fbacc   a
        nop
        nop
        ld      sint26v, a      ; 11 - software interrupt #26
        fbacc   a
        nop
        nop
        ld      sint27v, a      ; 12 - external interrupt #27
        fbacc   a
        nop
        nop
        ld      sint28v, a      ; 13 - software interrupt #28
        fbacc   a
        nop
        nop
        ld      sint29v, a      ; 14 - software interrupt #29
        fbacc   a
        nop
        nop
        ld      sint30v, a      ; 15 - software interrupt #30
        fbacc   a
        nop
        nop
        b       0ff00h          ; 16 - external interrupt INT0
        nop
        nop
        call    0ff01h          ; 17 - external interrupt INT1
        idle    1
        nop
        idle    2               ; 18 - external interrupt INT2
        nop
        nop
        nop
        ld      tintv, a        ; 19 - timmer interrupt
        fbacc   a
        nop
        nop
        ld      brint0v, a      ; 20 - BSP0 receive interrupt
        fbacc   a
        nop
        nop
        ld      bxint0v, a      ; 21 - BSP0 transmit interrupt
        fbacc   a
        nop
        nop
        ld      trntv, a        ; 22 - TDM port receive interrupt
        fbacc   a
        nop
        nop
        ld      txntv, a        ; 23 - TDM port transmit interrupt
        fbacc   a
        nop
        nop
        ld      int3v, a        ; 24 - external interrupt INT3
        fbacc   a
        nop
        nop
        ld      hpintv, a       ; 25 - HPI interrupt
        fbacc   a
        nop
        nop
        ld      brint1v, a      ; 26 - BSP1 receive interrupt
        fbacc   a
        nop
        nop
        ld      bxint1v, a      ; 27 - BSP1 transmit interrupt
        fbacc   a
        nop
        nop
        .end
